/// <reference types="bn.js" />
import { Zilliqa, BN } from "@zilliqa-js/zilliqa";
import { Transaction } from "@zilliqa-js/account";
import { ByStr20, ByStr33, Uint128 } from "./signable";
export declare const MAGENTA = "\u001B[35m%s\u001B[0m";
/**
 * create an account using Zilliqa
 */
export declare function createAccount(): {
    account: import("@zilliqa-js/account").Account;
    address: ByStr20;
    pubkey: ByStr33;
};
/**
 * get min gas price return already as BN
 */
export declare function getMinGasPrice(zil: Zilliqa): Promise<{
    gasPrice: BN;
}>;
export declare type TXLog = (t: Transaction, msg: string) => void;
/**
 * resolvers are the core of any sdk
 * they allow the sdk to be portable by giving functions that resolve to different
 * zilliqa network configurations
 * such as testnet mainnet isolated server
 * beyond that you can provide different submitter keys for contract calls and deploys
 */
export interface SDKResolvers {
    getZil: (requireSigner?: boolean) => Promise<{
        zil: Zilliqa;
        teardown: () => Promise<void>;
    }>;
    getVersion: () => number;
    getNetworkName: () => string;
    txLog?: TXLog;
}
/**
 * Send zil to an account convinience method
 *
 * @param resolvers the sdk resolvers
 * @param to address to send the zil to
 * @param amt of zil to send
 * @returns the transaction if successful
 */
export declare function sendZIL(resolvers: SDKResolvers, to: ByStr20, amt: Uint128): Promise<Transaction>;
export declare function getBalance(getZil: SDKResolvers["getZil"], a: ByStr20): Promise<Uint128>;
