"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrapTokens = exports.escapeRegExp = void 0;
function escapeRegExp(text) {
    return text.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&");
}
exports.escapeRegExp = escapeRegExp;
function prepareWords(wordsIn) {
    var _w = wordsIn.map(function (_t) { return ({
        length: _t.source.length,
        regex: new RegExp("^" + _t.regex.source),
        source: _t.source,
    }); });
    _w.sort(function (a, b) { return b.length - a.length; });
    return _w;
}
function prepareSymbols(i) {
    var _w = i.map(function (_t) { return ({
        length: _t.length,
        regex: new RegExp("(" + escapeRegExp(_t) + ")", "g"),
    }); });
    _w.sort(function (a, b) { return b.length - a.length; });
    return _w;
}
function getAntiTest(t) {
    var anti = Array.from(new Set(t.map(function (i) { return i.source[0]; })));
    return new RegExp("^\\s*(" + anti.join("|") + ")");
}
function lexWords(code, _tokens) {
    var anti = getAntiTest(_tokens);
    // to accomodate regexes for the longest token +2
    var longestToken = _tokens[0].length + 2;
    var pad = new Array(longestToken).fill(" ").join("");
    code = " " + code + pad;
    var res = [];
    var lastToken = 0;
    var i = 0;
    var start = 0;
    windowLoop: while (i < code.length) {
        start = i;
        var end = i + longestToken;
        var chars = code.slice(start, end);
        // if this does not match none of the tokens will
        if (!chars.match(anti)) {
            i++;
            continue windowLoop;
        }
        for (var _i = 0, _tokens_1 = _tokens; _i < _tokens_1.length; _i++) {
            var token = _tokens_1[_i];
            var isMatch = chars.match(token.regex);
            if (isMatch) {
                // console.log(token.regex)
                // even if the previous char is matched
                // consider case like =>match then target would be
                // match... it would not connect!
                var jump = isMatch[0].trimEnd().length;
                i = i + jump;
                if (start != lastToken) {
                    res.push(code.slice(lastToken, start));
                }
                lastToken = i;
                res.push(token.source);
                continue windowLoop;
            }
        }
        i++;
    }
    return __spreadArray(__spreadArray([], res, true), [code.slice(lastToken)], false).map(function (t) { return t.trim(); })
        .filter(function (i) { return i != ""; });
}
var wrapTokens = function (tokens, symbols) { return function (code) {
    var _tokens = prepareWords(tokens);
    var _symbols = prepareSymbols(symbols);
    var first = _symbols.shift();
    if (!first) {
        throw new Error("Provide at least one symbol");
    }
    // make a first pass for the symbols
    var symbolSplit = _symbols.reduce(function (prev, cur) {
        return prev.flatMap(function (a) { return (symbols.includes(a) ? [a] : a.split(cur.regex)); });
    }, code.split(first.regex));
    if (!symbolSplit) {
        throw new Error("Error splitting symbols");
    }
    return symbolSplit.flatMap(function (i) { return lexWords(i, _tokens); });
}; };
exports.wrapTokens = wrapTokens;
//# sourceMappingURL=lex.js.map