"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateBindings = void 0;
var deploy_1 = require("./deploy");
var transition_1 = require("./transition");
var boilerplate_1 = require("./boilerplate");
var stateGetter_1 = require("./stateGetter");
var getters_1 = require("./getters");
var shared_1 = require("./shared");
var __1 = require("../../..");
function generateBindings(abi, code, options) {
    var a = JSON.parse(abi);
    (0, getters_1.setABI)(a);
    var sourceCodeHash = new __1.ScillaString(code).toHash();
    var deployCode = (0, deploy_1.buildDeploy)(code, sourceCodeHash);
    var transitions = (0, transition_1.buildTransitions)(code, options.makeSigners);
    var boostZilPath = options["custom-boost-zil-path"]
        ? options["custom-boost-zil-path"]
        : "boost-zil";
    (0, getters_1.addImport)("import { BN, Long } from \"@zilliqa-js/util\";", "import { Transaction, TxParams } from \"@zilliqa-js/account\";", "import { Contract } from \"@zilliqa-js/contract\";", "import * as T from \"" + boostZilPath + "\"", "import { signTransition } from \"" + boostZilPath + "\";", "import { Zilliqa } from \"@zilliqa-js/zilliqa\";", "import {ContractSubStateQueryCast, partialState } from \"" + boostZilPath + "\"");
    var imports = (0, getters_1.getImports)();
    return {
        sdkCode: "\n" + imports.join("\n") + "\n\n/**\n * this string is the signature of the hash of the source code\n * that was used to generate this sdk\n */\nexport const contractSignature = \"hash_" + sourceCodeHash + "\";\nconst sig: {\n  contractSignature: \"hash_" + sourceCodeHash + "\";\n} = { contractSignature };\n\nexport const code = `\n(* sourceCodeHash=" + sourceCodeHash + " *)\n(* sourceCodeHashKey=hash_" + sourceCodeHash + " *)\n" + (0, shared_1.removeComments)(code) + "`\n\n" + boilerplate_1.boilerplate + "\n\nexport const " + a.contract_info.vname + " = (resolvers: SDKResolvers) => {\n  const logUrl = (id: string, msg: string) => {\n    const network = getNetworkName();\n    console.log(MAGENTA, msg + \" \uD83D\uDD25\");\n    if (network == \"mainnet\" || network == \"testnet\") {\n      const url = `https://viewblock.io/zilliqa/tx/0x${id}?network=${network}`;\n      console.log(CYAN, url);\n    }\n  };\n  const zilpayLog = (t: Transaction, msg: string) => {\n    console.log(t);\n    //@ts-ignore\n    const id = t.ID;\n    logUrl(id as string, msg);\n  };\n  const nodeLog = (t: Transaction, msg: string) => {\n    const id = t.id;\n    logUrl(id as string, msg);\n    const receipt = t.getReceipt();\n    if (receipt) {\n      if (receipt.success) {\n        console.log(GREEN, \"Success.\");\n      } else {\n        console.log(RED, \"Failed.\");\n        if (receipt.exceptions) {\n          Object.entries(receipt.exceptions).map(([k, v]) => {\n            console.log(RED, v);\n          });\n        }\n      }\n      if (receipt.event_logs) {\n        const events = receipt.event_logs as {\n          _eventname: string;\n          address: string;\n          params: { value: string; vname: string }[];\n        }[];\n        if (events.length != 0) {\n          console.log(CYAN, `Events\uD83D\uDD75\uFE0F\u200D\u2640\uFE0F`);\n          events.forEach((e) => {\n            console.log(CYAN, `${e._eventname}`);\n            e.params.forEach((p) =>\n           {\n             console.log(CYAN, `${p.vname}: `)\n             console.log(MAGENTA, `${JSON.stringify(p.value, null, 2)}`)\n            }\n            );\n          });\n        }\n      }\n    }\n  };\n  const defaultTxLog = (t: Transaction, msg: string) => {\n    if (thereIsZilPay()) {\n      zilpayLog(t, msg);\n    } else {\n      nodeLog(t, msg);\n    }\n  };\n    const {\n      getZil,\n      getVersion,\n      getNetworkName,\n    } = resolvers;\n    const txLink = resolvers.txLog ? resolvers.txLog : defaultTxLog;\n    \n    return {\n  async balance(a: T.ByStr20) {\n    const res = await getZil();\n    const bal = await res.zil.blockchain.getBalance(a.toSend());\n    await res.teardown();\n    return new T.Uint128(bal.result.balance);\n  },\n\n  /**\n   * will try to send a transaction to the contract\n   * @warning WILL NOT THROW ERRORS IF CONTRACT SIGNATURES ARE INVALID\n   */\n  dangerousFromJSONDeploy: dangerousFromJSONDeploy(txLink)(resolvers),\n\n  /**\n   * will try to send a transaction to the contract\n   * @warning WILL NOT THROW ERRORS IF CONTRACT SIGNATURES ARE INVALID\n   */\n  dangerousFromJSONCall: dangerousFromJSONCall(txLink)(resolvers),\n\n" + deployCode + ",\n\n" + (0, stateGetter_1.stateGetter)() + "\n\n  /**\n   * interface for scilla contract with source code hash:\n   * " + sourceCodeHash + "\n  */\n  calls: (a: T.ByStr20) => (gasLimit: Long) => {\n    const signer = signTransition(a);\n    return {" + transitions.built + "}\n  }\n}\n}\n",
        documentation: transitions.documentation,
    };
}
exports.generateBindings = generateBindings;
//# sourceMappingURL=index.js.map