"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTypescriptType = void 0;
var zil_accept_1 = require("../zil-accept");
function getCustomADT(name, abi) {
    var found = abi.contract_info.ADTs.find(function (a) { return a.tname == name; });
    if (!found) {
        throw new Error("Couldn't find custom user ADT " + name);
    }
    return found;
}
var getTypescriptType = function (scillaType, contractName, contractAddressVarName, abi, varname) {
    var statementsToAdd = [];
    function translateScillaTypeToTypescriptType(lexed) {
        if (lexed.length == 0) {
            return "";
        }
        var t = lexed[0].w;
        if (t.startsWith(contractName)) {
            statementsToAdd.push(varname + ".setContractAddress(" + contractAddressVarName + ");");
            var adtName = t.replace(contractName + ".", "");
            statementsToAdd.push(varname + ".setADTname(\"" + adtName + "\");");
            var res = parseCustomADT(lexed);
            return res;
        }
        if (t.startsWith("(") || t.startsWith(")")) {
            lexed.shift();
            return translateScillaTypeToTypescriptType(lexed);
        }
        if (t.startsWith("List")) {
            var res = parseList(lexed);
            return res;
        }
        if (t.startsWith("Pair")) {
            var res = parsePair(lexed);
            return res;
        }
        if (t.startsWith("Uint128")) {
            return "T.Uint128";
        }
        if (t.startsWith("Uint32")) {
            return "T.Uint32";
        }
        if (t.startsWith("Uint256")) {
            return "T.Uint256";
        }
        if (t.startsWith("ByStr20")) {
            return "T.ByStr20";
        }
        if (t.startsWith("ByStr33")) {
            return "T.ByStr33";
        }
        if (t.startsWith("ByStr64")) {
            return "T.ByStr64";
        }
        if (t.startsWith("ByStr")) {
            return "T.ByStr";
        }
        if (t.startsWith("String")) {
            return "T.ScillaString";
        }
        if (t.startsWith("BNum")) {
            return "T.BNum";
        }
        if (t.startsWith("Uint")) {
            return "BN";
        }
        return "Value[\"value\"] | {\n            constructor: string;\n            argtypes: string[];\n            arguments: Value[] | string[];\n        }[] | string[]";
    }
    function parseList(list) {
        if (list.length == 0) {
            return "";
        }
        var first = list.shift();
        if (first.w == "List") {
            return "T.List<" + translateScillaTypeToTypescriptType(list) + ">";
        }
        else {
            return translateScillaTypeToTypescriptType(list);
        }
    }
    function parsePair(list) {
        if (list.length == 0) {
            return "";
        }
        var first = list.shift();
        if (first.w == "Pair") {
            var fst = list;
            var snd = list.slice(4);
            return "T.Pair<" + translateScillaTypeToTypescriptType(fst) + "," + translateScillaTypeToTypescriptType(snd) + ">";
        }
        else {
            return translateScillaTypeToTypescriptType(list);
        }
    }
    function parseCustomADT(list) {
        if (list.length == 0) {
            return "";
        }
        var first = list.shift();
        if (first.w.startsWith(contractName + ".")) {
            var adt = getCustomADT(first.w, abi);
            var argtypes = adt.tmap[0].argtypes.map(function (a) {
                return (0, zil_accept_1.customLex)(a, ["(", ")", ",", "{", "}"]);
            });
            return "T.CustomADT<[" + argtypes
                .map(translateScillaTypeToTypescriptType)
                .join(",") + "]>";
        }
        else {
            return translateScillaTypeToTypescriptType(list);
        }
    }
    var lexed = (0, zil_accept_1.customLex)(scillaType, ["(", ")", ",", "{", "}"]);
    return {
        typescriptType: translateScillaTypeToTypescriptType(lexed),
        statementsToAdd: statementsToAdd,
    };
};
exports.getTypescriptType = getTypescriptType;
//# sourceMappingURL=getTypescriptType.js.map