"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildTransitions = void 0;
var shared_1 = require("./shared");
var zil_accept_1 = require("./zil-accept");
var zil_doc_1 = require("./zil-doc");
var getters_1 = require("./getters");
var setup_1 = require("../binder2/setup");
var parser_1 = require("../binder2/parser");
function getZilFlow(acceptsZil) {
    if (acceptsZil) {
        return {
            zilAmountParam: "amount: T.Uint128,",
            zilAmountVal: "amount.value",
        };
    }
    return { zilAmountParam: "", zilAmountVal: "new BN(0)" };
}
function buildTransition(trans, lexed, makeSigners, accepts) {
    var acceptsZil = accepts[trans.vname];
    var documentation = (0, zil_doc_1.zilDoc)(trans, lexed);
    var _a = getZilFlow(acceptsZil), zilAmountParam = _a.zilAmountParam, zilAmountVal = _a.zilAmountVal;
    var _b = (0, shared_1.getParamAST)(trans.params, (0, getters_1.getABI)(), "a"), ast = _b.paramAST, statementsToAdd = _b.statementsToAdd;
    var signers = makeSigners
        ? "/**\n * returns a signer that given an account will sign transition params in such manner:\n * ...params, nonce, transition name, contract address\n * */\nsign_" + trans.vname + ": signer(\"" + trans.vname + "\"),"
        : "";
    var sig = ast.map(function (i) { return i.varName + ": " + i.typescriptType; }).join(",\n");
    var param = "[\n    " + ast.map(function (i) { return (0, shared_1.getParam)(i.type, i.vname, i.varValue); }).join(",\n") + "\n    ]";
    return {
        transition: "\n" + signers + "\n" + trans.vname + ": (" + zilAmountParam + " " + sig + ")=> {\n" + statementsToAdd.join("\n") + "\n  const transactionData = {\n    isDeploy: false,\n    ...sig,\n    contractAddress: a.toSend(),\n    contractTransitionName: `" + trans.vname + "`,\n    data: " + param + ",\n    amount: " + zilAmountVal + ".toString()\n  }; \n  return {\n/**\n * get data needed to perform this transaction\n * */\ntoJSON: () => transactionData,\n/**\n * send the transaction to the blockchain\n * */\nsend: async () => dangerousFromJSONCall(txLink)(resolvers)(transactionData, gasLimit), \n  }\n},\n",
        documentation: documentation,
    };
}
function buildTransitions(code, makeSigners) {
    var lexed = (0, zil_accept_1.lexer)(code);
    var transitions = (0, getters_1.getABI)()
        .contract_info.transitions //filter out callbacks
        .filter(function (t) { return !t.vname.includes("CallBack"); })
        .filter(function (t) { return !t.vname.includes("RecipientAcceptTransfer"); })
        .filter(function (t) { return !t.vname.includes("RecipientAcceptTransferFrom"); });
    /**
     * from the better end to end lexer and exhaustive parser
     */
    var lexed2 = (0, setup_1.lexScilla)(code);
    var code2 = (0, parser_1.justBody)(lexed2);
    var transitionAccepts = (0, parser_1.justTransitions)(code2).reduce(function (p, c) {
        p[c.name] = !!c.accepts;
        return p;
    }, {});
    var processed = transitions.map(function (t) {
        return buildTransition(t, lexed, makeSigners, transitionAccepts);
    });
    var built = processed.map(function (p) { return p.transition; }).join("\n");
    var documentation = (0, zil_doc_1.getZilDocHeader)((0, getters_1.getABI)().contract_info.vname) +
        processed.map(function (p) { return p.documentation; }).join("\n\n");
    return {
        documentation: documentation,
        built: built,
    };
}
exports.buildTransitions = buildTransitions;
//# sourceMappingURL=transition.js.map