"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildBind = void 0;
var abi_1 = require("./abi");
var path_1 = require("path");
var fs_1 = require("fs");
var prettier_1 = require("prettier");
var shared_1 = require("./shared");
var create_hash_1 = __importDefault(require("create-hash"));
function sha256String(s) {
    var sha = (0, create_hash_1.default)("sha256");
    sha.update(s);
    return "0x" + sha.digest().toString("hex");
}
function createDirIfNotExists(dir) {
    !(0, fs_1.existsSync)(dir) && (0, fs_1.mkdirSync)(dir, { recursive: true });
}
function getCachedABI(abiPath, sourceCodeHash) {
    if ((0, fs_1.existsSync)(abiPath)) {
        var previousAbi = JSON.parse((0, fs_1.readFileSync)(abiPath, "utf-8"));
        if (previousAbi.source_hash == sourceCodeHash) {
            console.log(shared_1.CYAN, "\uD83E\uDDE0 Contract source code didn't change, not getting the ABI, just building the SDK.");
            return previousAbi;
        }
    }
    return undefined;
}
function buildBind(contractPath, options) {
    return __awaiter(this, void 0, void 0, function () {
        var scillaContract, contractFileName, contractDir, codePath, abiPath, bindPath, buildDirectory, documentationPath, code, sourceCodeHash, cachedABI, abi, _a, contractName, uuidRegex, stringified, _b, sdkCode, documentation, bindings, e_1;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    _c.trys.push([0, 4, , 5]);
                    scillaContract = contractPath;
                    contractFileName = (0, path_1.basename)(contractPath);
                    console.time(contractFileName);
                    contractDir = (0, path_1.dirname)(scillaContract);
                    codePath = (0, path_1.resolve)(contractDir, scillaContract);
                    abiPath = (0, path_1.resolve)(contractDir, "./build/abi.json");
                    bindPath = (0, path_1.resolve)(contractDir, "./build/bind.ts");
                    buildDirectory = (0, path_1.resolve)(contractDir, "./build/");
                    documentationPath = (0, path_1.resolve)(contractDir, "./README.md");
                    createDirIfNotExists(buildDirectory);
                    code = (0, fs_1.readFileSync)(codePath, "utf-8");
                    sourceCodeHash = sha256String(code);
                    cachedABI = getCachedABI(abiPath, sourceCodeHash);
                    if (!cachedABI) return [3 /*break*/, 1];
                    _a = cachedABI;
                    return [3 /*break*/, 3];
                case 1: return [4 /*yield*/, (0, abi_1.getABI)(code, sourceCodeHash)];
                case 2:
                    _a = _c.sent();
                    _c.label = 3;
                case 3:
                    abi = _a;
                    contractName = abi.contract_info.vname;
                    uuidRegex = /\b[0-9a-f]{8}\b-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-\b[0-9a-f]{12}./g;
                    stringified = JSON.stringify(abi, null, 2).replace(uuidRegex, contractName + ".");
                    (0, fs_1.writeFileSync)(abiPath, stringified);
                    _b = (0, abi_1.generateBindings)(stringified, code, options), sdkCode = _b.sdkCode, documentation = _b.documentation;
                    bindings = sdkCode;
                    try {
                        bindings = (0, prettier_1.format)(sdkCode, {
                            parser: "babel-ts",
                            insertPragma: true,
                        });
                    }
                    catch (e) {
                        console.warn("Failed to format code, it means that there is a type error, inspect the output!");
                    }
                    (0, fs_1.writeFileSync)(bindPath, bindings);
                    (0, fs_1.writeFileSync)(documentationPath, documentation);
                    console.timeEnd(contractFileName);
                    return [3 /*break*/, 5];
                case 4:
                    e_1 = _c.sent();
                    console.error(e_1);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/];
            }
        });
    });
}
exports.buildBind = buildBind;
//# sourceMappingURL=build.js.map