"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsonConfigMaker = void 0;
var fs_1 = require("fs");
function mergeDeep(target, source) {
    var _a, _b;
    if (typeof source == "object") {
        for (var key in source) {
            if (typeof source[key] == "object") {
                if (!target[key])
                    Object.assign(target, (_a = {}, _a[key] = {}, _a));
                mergeDeep(target[key], source[key]);
            }
            else {
                Object.assign(target, (_b = {}, _b[key] = source[key], _b));
            }
        }
    }
}
/**
 * Allows to create a permament config as a json file that
 * conforms to a specific ts interface
 * @param pathToConfig the absolute path to your config
 * @returns an object to interact with your config, update it and get it
 * and mock its values for testing purposes
 */
var jsonConfigMaker = function (pathToConfig) {
    if (!(0, fs_1.existsSync)(pathToConfig)) {
        console.warn("Created new config file, since none was found");
        (0, fs_1.writeFileSync)(pathToConfig, JSON.stringify({}));
    }
    var getConfigBase = function () { return JSON.parse((0, fs_1.readFileSync)(pathToConfig, "utf-8")); };
    var temp = getConfigBase();
    function getConfig(mock) {
        if (mock) {
            return temp;
        }
        return getConfigBase();
    }
    /**
     * @param mock don't update the file, but update the local state
     */
    return function (mock) { return ({
        updateConfig: function (toMerge) {
            var config = getConfig(mock);
            mergeDeep(config, toMerge);
            if (!mock) {
                (0, fs_1.writeFileSync)(pathToConfig, JSON.stringify(config, null, 2));
            }
            else {
                temp = config;
            }
        },
        getConfig: function () { return getConfig(mock); },
    }); };
};
exports.jsonConfigMaker = jsonConfigMaker;
//# sourceMappingURL=local-config-maker.js.map