import { Signable } from "./shared";
declare abstract class ByStrSignable extends Signable {
    value: string;
    constructor(v: string);
}
declare abstract class AnyByStr extends ByStrSignable {
    constructor(v: string);
    toHash(): string;
    toSend(): string;
}
/**
 * the bystr20 type class, it manages the formatting of the address
 * to make sure that it always goes through in the on chain transactions
 */
export declare class ByStr20 extends ByStrSignable {
    type: string;
    static zeroVal: string;
    constructor(v: string);
    static fromPubKey(key: string): ByStr20;
    static areEqual(first: string | ByStr20, second: string | ByStr20): boolean;
    toHash(): string;
    /**
     * it returns the address without the 0x
     * it is used for JSON RPC requests to the Zilliqa blockchain API when giving it
     * address of the smart contract for instance to GetSmartContractState
     */
    noPrefixed(): string;
    /**
     * JSON RPC API requires lower case addresses for the indices of GetSmartContractSubState
     */
    lowerCase(): string;
    toBech32(): string;
    /**
     * sometimes useful
     * @returns 0x0000000000000000000000000000000000000000
     */
    static zeroByStr20(): ByStr20;
    static isValid(s: string): boolean;
    toSend(): string;
}
export declare class ByStr33 extends AnyByStr {
    type: string;
}
export declare class ByStr64 extends AnyByStr {
    type: string;
}
export declare class ByStr extends AnyByStr {
    type: string;
}
export {};
