"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.ByStr = exports.ByStr64 = exports.ByStr33 = exports.ByStr20 = void 0;
var shared_1 = require("./shared");
var utill_1 = require("../utill");
var crypto_1 = require("@zilliqa-js/crypto");
var ByStrSignable = /** @class */ (function (_super) {
    __extends(ByStrSignable, _super);
    function ByStrSignable(v) {
        var _this = _super.call(this) || this;
        _this.value = v;
        return _this;
    }
    return ByStrSignable;
}(shared_1.Signable));
var AnyByStr = /** @class */ (function (_super) {
    __extends(AnyByStr, _super);
    function AnyByStr(v) {
        var _this = this;
        var toSet = v.startsWith("0x") ? v : "0x" + v;
        _this = _super.call(this, toSet) || this;
        return _this;
    }
    AnyByStr.prototype.toHash = function () {
        return (0, shared_1.sha256)(this.value.toLowerCase().replace("0x", ""));
    };
    AnyByStr.prototype.toSend = function () {
        return this.value.toLowerCase();
    };
    return AnyByStr;
}(ByStrSignable));
/**
 * the bystr20 type class, it manages the formatting of the address
 * to make sure that it always goes through in the on chain transactions
 */
var ByStr20 = /** @class */ (function (_super) {
    __extends(ByStr20, _super);
    function ByStr20(v) {
        var _this = _super.call(this, v) || this;
        _this.type = "ByStr20";
        return _this;
    }
    ByStr20.fromPubKey = function (key) {
        return new ByStr20((0, crypto_1.getAddressFromPublicKey)(key));
    };
    ByStr20.areEqual = function (first, second) {
        try {
            var f = first instanceof ByStr20 ? first : new ByStr20(first);
            var s = second instanceof ByStr20 ? second : new ByStr20(second);
            return f.toSend() == s.toSend();
        }
        catch (e) { }
        return false;
    };
    ByStr20.prototype.toHash = function () {
        return (0, shared_1.sha256)((0, utill_1.normaliseAddress)(this.value).replace("0x", ""));
    };
    /**
     * it returns the address without the 0x
     * it is used for JSON RPC requests to the Zilliqa blockchain API when giving it
     * address of the smart contract for instance to GetSmartContractState
     */
    ByStr20.prototype.noPrefixed = function () {
        return this.toSend().replace("0x", "");
    };
    /**
     * JSON RPC API requires lower case addresses for the indices of GetSmartContractSubState
     */
    ByStr20.prototype.lowerCase = function () {
        return this.toSend().toLowerCase();
    };
    ByStr20.prototype.toBech32 = function () {
        return (0, crypto_1.toBech32Address)(this.toSend());
    };
    /**
     * sometimes useful
     * @returns 0x0000000000000000000000000000000000000000
     */
    ByStr20.zeroByStr20 = function () {
        return new ByStr20(ByStr20.zeroVal);
    };
    ByStr20.isValid = function (s) {
        try {
            (0, utill_1.normaliseAddress)(s);
            return true;
        }
        catch (e) {
            return false;
        }
    };
    ByStr20.prototype.toSend = function () {
        if (this.value == ByStr20.zeroVal) {
            return this.value;
        }
        var res = (0, utill_1.normaliseAddress)(this.value);
        if (!res) {
            throw new Error("spooky undefined address");
        }
        return res;
    };
    ByStr20.zeroVal = "0x0000000000000000000000000000000000000000";
    return ByStr20;
}(ByStrSignable));
exports.ByStr20 = ByStr20;
var ByStr33 = /** @class */ (function (_super) {
    __extends(ByStr33, _super);
    function ByStr33() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = "ByStr33";
        return _this;
    }
    return ByStr33;
}(AnyByStr));
exports.ByStr33 = ByStr33;
var ByStr64 = /** @class */ (function (_super) {
    __extends(ByStr64, _super);
    function ByStr64() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = "ByStr64";
        return _this;
    }
    return ByStr64;
}(AnyByStr));
exports.ByStr64 = ByStr64;
var ByStr = /** @class */ (function (_super) {
    __extends(ByStr, _super);
    function ByStr() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = "ByStr";
        return _this;
    }
    return ByStr;
}(AnyByStr));
exports.ByStr = ByStr;
//# sourceMappingURL=bystr.js.map