/// <reference types="bn.js" />
import { BN } from "@zilliqa-js/util";
import { Signable } from "./shared";
import Big from "big.js";
declare abstract class UintSignable extends Signable {
    value: BN;
    constructor(v: string | BN);
    asBN(): BN;
    asBig(): Big;
}
export declare class Uint32 extends UintSignable {
    type: string;
    constructor(v: string | BN);
    toHash(): string;
    toSend(): string;
}
interface TokenInfo {
    decimals: number;
    precision: number;
    symbol: string;
}
export declare class Uint128 extends UintSignable {
    type: string;
    private tokenInfo;
    setTokenInfo(info: TokenInfo): void;
    getTokenInfo(): TokenInfo;
    /**
     * throws error if tokenInfo is not set
     * returns a readable frontend formatted string
     */
    getReadable(): string;
    /**
     * makes a Uint128 with the same token info
     */
    static fromStringUint128(other: Uint128, newVal: string): Uint128;
    /**
     * makes a Uint128 with the same token info from a fraction
     */
    static fromFractionUint128(other: Uint128, newFraction: string): Uint128;
    constructor(v: string | BN);
    toHash(): string;
    toSend(): string;
    static getRandom(): Uint128;
    static zil(v: string | BN): Uint128;
    /**
     *
     * @param fraction the string representing a fraction delimited with a "."
     * example: 0.04
     * @param decimals the number of decimals the target number represented has
     * example: 0.04 decimals 12 would result in:
     * 40 000 000 000
     */
    static fromFraction(fraction: string, decimals: number): Uint128;
    /**
     *
     * @param str the string or Uint128 from the blockchain
     * @param decimals the decimals in the string that represents some token§
     * @param precision the precision of the fraction that the output string represents
     */
    static fromStringtoFraction(str: string, decimals: number, precision?: number): string;
}
export declare class Uint256 extends UintSignable {
    type: string;
    constructor(v: string | BN);
    toHash(): string;
    toSend(): string;
}
export {};
