"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Uint256 = exports.Uint128 = exports.Uint32 = void 0;
var util_1 = require("@zilliqa-js/util");
var shared_1 = require("./shared");
var randombytes_1 = __importDefault(require("randombytes"));
var big_js_1 = __importDefault(require("big.js"));
var UintSignable = /** @class */ (function (_super) {
    __extends(UintSignable, _super);
    function UintSignable(v) {
        var _this = _super.call(this) || this;
        _this.value = new util_1.BN(v);
        return _this;
    }
    UintSignable.prototype.asBN = function () {
        return this.value;
    };
    UintSignable.prototype.asBig = function () {
        return new big_js_1.default(this.value.toString());
    };
    return UintSignable;
}(shared_1.Signable));
var Uint32 = /** @class */ (function (_super) {
    __extends(Uint32, _super);
    function Uint32(v) {
        var _this = _super.call(this, v) || this;
        _this.type = "Uint32";
        return _this;
    }
    Uint32.prototype.toHash = function () {
        return (0, shared_1.sha256)(this.value.toString("hex", 8));
    };
    Uint32.prototype.toSend = function () {
        return this.value.toString();
    };
    return Uint32;
}(UintSignable));
exports.Uint32 = Uint32;
var Uint128 = /** @class */ (function (_super) {
    __extends(Uint128, _super);
    function Uint128(v) {
        var _this = _super.call(this, v) || this;
        _this.type = "Uint128";
        return _this;
    }
    Uint128.prototype.setTokenInfo = function (info) {
        this.tokenInfo = info;
    };
    Uint128.prototype.getTokenInfo = function () {
        if (!this.tokenInfo) {
            throw new Error("Token info not set");
        }
        return this.tokenInfo;
    };
    /**
     * throws error if tokenInfo is not set
     * returns a readable frontend formatted string
     */
    Uint128.prototype.getReadable = function () {
        var info = this.getTokenInfo();
        return Uint128.fromStringtoFraction(this.toSend(), info.decimals, info.precision);
    };
    /**
     * makes a Uint128 with the same token info
     */
    Uint128.fromStringUint128 = function (other, newVal) {
        var u = new Uint128(newVal);
        u.setTokenInfo(other.getTokenInfo());
        return u;
    };
    /**
     * makes a Uint128 with the same token info from a fraction
     */
    Uint128.fromFractionUint128 = function (other, newFraction) {
        var info = other.getTokenInfo();
        var u = this.fromFraction(newFraction, info.decimals);
        u.setTokenInfo(info);
        return u;
    };
    Uint128.prototype.toHash = function () {
        return (0, shared_1.sha256)(this.value.toString("hex", 32));
    };
    Uint128.prototype.toSend = function () {
        return this.value.toString();
    };
    Uint128.getRandom = function () {
        return new Uint128(new util_1.BN((0, randombytes_1.default)(16).toString("hex"), "hex"));
    };
    Uint128.zil = function (v) {
        var z = new Uint128(util_1.units.toQa(v, util_1.units.Units.Zil));
        z.setTokenInfo({ decimals: 12, precision: 3, symbol: "ZIL" });
        return z;
    };
    /**
     *
     * @param fraction the string representing a fraction delimited with a "."
     * example: 0.04
     * @param decimals the number of decimals the target number represented has
     * example: 0.04 decimals 12 would result in:
     * 40 000 000 000
     */
    Uint128.fromFraction = function (fraction, decimals) {
        var frac = new big_js_1.default(fraction);
        var target = frac.mul(new big_js_1.default(10).pow(decimals));
        return new Uint128(target.toFixed(0));
    };
    /**
     *
     * @param str the string or Uint128 from the blockchain
     * @param decimals the decimals in the string that represents some token§
     * @param precision the precision of the fraction that the output string represents
     */
    Uint128.fromStringtoFraction = function (str, decimals, precision) {
        if (precision === void 0) { precision = 3; }
        var b = new big_js_1.default(str).div(new big_js_1.default(10).pow(decimals));
        return b.toFixed(precision);
    };
    return Uint128;
}(UintSignable));
exports.Uint128 = Uint128;
var Uint256 = /** @class */ (function (_super) {
    __extends(Uint256, _super);
    function Uint256(v) {
        var _this = _super.call(this, v) || this;
        _this.type = "Uint256";
        return _this;
    }
    Uint256.prototype.toHash = function () {
        return (0, shared_1.sha256)(this.value.toString("hex", 64));
    };
    Uint256.prototype.toSend = function () {
        return this.value.toString();
    };
    return Uint256;
}(UintSignable));
exports.Uint256 = Uint256;
//# sourceMappingURL=uint.js.map