import { Account, Transaction } from "@zilliqa-js/account";
import { Zilliqa } from "@zilliqa-js/zilliqa";
import { ByStr20, ByStr33 } from "../signable";
declare type TXLog = (t: Transaction, msg: string) => void;
interface SDKResolvers {
    getZil: (requireSigner?: boolean) => Promise<{
        zil: Zilliqa;
        teardown: () => Promise<void>;
    }>;
    getVersion: () => number;
    getNetworkName: () => string;
    txLog?: TXLog;
}
export declare const sleep: (milis: number) => Promise<void>;
/**
 * isolated server object
 * has all of the config to do testing, including a funded private key
 * make sure to .kill() the server after you are done
 */
export declare const isolatedServer: {
    mockConfig: {
        networkname: string;
        version: string;
        nodeurl: string;
    };
    submitterPrivateKey: string;
    otherKeys: {
        second: {
            bystr20: string;
            privateKey: string;
        };
    };
    start: () => Promise<void>;
    kill: () => Promise<void>;
};
/**
 * isolated server resolvers from account
 */
export declare function getResolversFromAccount(a: Account): SDKResolvers;
/**
 *
 * @returns a funded account on the isolated server
 */
export declare function getFundedAccount(): {
    adminAcc: Account;
    adminAddr: ByStr20;
    adminPubKey: ByStr33;
};
export {};
