declare const _default: "scilla_version 0\n\nimport IntUtils\n\nlibrary SlotMachine\ntype Error =\n| NotAuthorized\n| NoStagedAdmin\n| NotEnoughZILSent\n| InvalidSignature\n| CannotClaimMoreThanMax\n| UnknownRewardTier\n| DiscountTooBig\n| InvalidCheque\n\nlet make_error =\nfun (result: Error) =>\nlet result_code =\nmatch result with\n| NotAuthorized => Int32 -1\n| NoStagedAdmin => Int32 -2\n| NotEnoughZILSent => Int32 -3\n| InvalidSignature => Int32 -4\n| CannotClaimMoreThanMax => Int32 -5\n| UnknownRewardTier => Int32 -6\n| DiscountTooBig => Int32 -7\n| InvalidCheque => Int32 -8\nend\nin\n{ _exception: \"Error\"; code: result_code }\n\nlet one_msg = \n    fun(msg: Message) =>\n    let nil_msg = Nil{Message} in Cons{Message} msg nil_msg\n\nlet zero = Uint128 0\nlet one = Uint128 1\nlet zeroByStr0 = 0x\nlet zeroByStr20 = 0x0000000000000000000000000000000000000000\nlet zeroByStr = builtin to_bystr zeroByStr0\nlet discount_denominator = Uint128 10000\nlet true = True\n\ntype Uint128Pair =\n| Uint128Pair of Uint128 Uint128\nlet uint128_to_uint256: Uint128 -> Uint256 =\n  fun (x: Uint128) =>\n    let ox256 = builtin to_uint256 x in\n      match ox256 with\n      | None =>\n        (* this never happens, hence we throw a division by zero exception just in case *)\n        let zero = Uint256 0 in\n        builtin div zero zero\n      | Some x256 => x256\n      end\n(* Compute \"(x * y) / z\" with protection against integer overflows *)\nlet muldiv: Uint128 -> Uint128 -> Uint128 -> Uint128 =\n    fun (x: Uint128) =>\n    fun (y: Uint128) =>\n    fun (z: Uint128) =>\n      let x256 = uint128_to_uint256 x in\n      let y256 = uint128_to_uint256 y in\n      let z256 = uint128_to_uint256 z in\n      let x_mul_y256 = builtin mul x256 y256 in\n      let res256 = builtin div x_mul_y256 z256 in\n      let ores128 = builtin to_uint128 res256 in\n      match ores128 with\n      | None =>\n        (* this must never happen, hence we throw an integer overflow exception *)\n        let max_uint128 = Uint128 340282366920938463463374607431768211455 in\n        let fourtytwo128 = Uint128 42 in\n        builtin mul max_uint128 fourtytwo128\n      | Some res128 =>\n        res128\n      end\n\n(* calculate amounts without comission *)\nlet sub_percentage_of_amt: Uint128 -> Uint128 -> Uint128Pair =\n    fun (amount: Uint128) => fun (commission_numerator: Uint128) =>\n    let commission = muldiv amount commission_numerator discount_denominator in\n    let amount_sub_commission = builtin sub amount commission in\n    let res = Uint128Pair amount_sub_commission commission in\n    res\n\ntype SpinClaim =\n| SpinClaim of Uint128 Uint128 Uint128 ByStr64\n\ntype Signable =\n| Signable of ByStr20 Uint128 Uint128 Uint128 ByStr20\n\ntype HashableArg =\n| B of ByStr20\n| U of Uint128\n\nlet hash_signable_arg: HashableArg -> ByStr32 =\n    fun(s: HashableArg)=> \n    match s with\n    | B a => builtin sha256hash a\n    | U a => builtin sha256hash a\n    end\n\nlet hash_concat: ByStr -> HashableArg -> ByStr =\n    fun(prefix: ByStr) => fun(suffix: HashableArg) =>\n    let bystr_32 = hash_signable_arg suffix in\n    let bystr = builtin to_bystr bystr_32 in\n    builtin concat prefix bystr\n\n\nlet signable_to_cheque_hash: Signable -> ByStr =\n    fun(args: Signable) =>\n        match args with\n            | Signable b0 u0 u1 u2 b1 =>\n            let c0 = let a0 = B  b0 in hash_concat zeroByStr a0 in\n            let c1 = let a1 = U  u0 in hash_concat        c0 a1 in\n            let c2 = let a2 = U  u1 in hash_concat        c1 a2 in\n            let c3 = let a3 = U  u2 in hash_concat        c2 a3 in\n            let c4 = let a4 = B  b1 in hash_concat        c3 a4 in\n                c4\n        end\n\nlet optional_uint128: Option Uint128 -> Uint128 =\n    fun(o: Option Uint128) =>\n    match o with | None => zero | Some s => s end\n\nlet empty_uint128_uint128_map = Emp Uint128 Uint128\n\ncontract SlotMachine(\n    init_admin_pubkey: ByStr33,\n    init_fee_addr: ByStr20,\n    init_ticket_price: Uint128,\n    init_fee_cut: Uint128\n)\n\nfield players_spins: Map ByStr20 Uint128 = Emp ByStr20 Uint128\nfield players_claimed:  Map ByStr20 Uint128 = Emp ByStr20 Uint128\nfield admin: ByStr20 = builtin schnorr_get_address init_admin_pubkey\nfield admin_pubkey: ByStr33 = init_admin_pubkey \nfield staging_admin_pubkey: Option ByStr33 = None {ByStr33}\n\n(* ex 0 to 6 and corresponding reward amt *)\nfield win_tiers: Map Uint128 Uint128 = empty_uint128_uint128_map\nfield fee_addr: ByStr20 = init_fee_addr\nfield ticket_price: Uint128 = init_ticket_price\nfield fee_cut: Uint128 = init_fee_cut\n\nfield tmp_ticket_amt: Uint128 = zero\n(* hashes of cheque hashes here are invalid *)\nfield void_cheques: Map ByStr32 Bool = Emp ByStr32 Bool\nfield tmp_target: ByStr20 = zeroByStr20\n\nprocedure ThrowError(err: Error)\n    e = make_error err;\n    throw e\nend\nprocedure IsAdmin()\n    tmp <- admin;\n    is_admin = builtin eq tmp _sender;\n    match is_admin with\n    | False => e = NotAuthorized; ThrowError e\n    | True =>\n    end \nend\nprocedure SendZil(beneficiary: ByStr20, amt: Uint128)\n    msg = let m = { \n        _tag: \"AddFunds\";\n        _recipient: beneficiary;\n        _amount: amt\n    } in one_msg m;\n    send msg\nend\nprocedure DoVoidCheque(cheque_hash: ByStr)\n    hash = builtin sha256hash cheque_hash;\n    void_cheques[hash] := true\nend\nprocedure AssertChequeHashValid(cheque_hash: ByStr)\n    hash = builtin sha256hash cheque_hash;\n    cheque_invalid <- exists void_cheques[hash];\n    match cheque_invalid with \n        | False =>\n        | True => e = InvalidCheque; ThrowError e\n    end\nend\n\ntransition SetStagedAdmin(staged: ByStr33)\n    IsAdmin;\n    opt_staged = Some {ByStr33} staged;\n    staging_admin_pubkey := opt_staged\nend\ntransition ClaimStagedAdmin()\n    option_staged <- staging_admin_pubkey;\n    match option_staged with\n    | None => e = NoStagedAdmin; ThrowError e\n    | Some staged_pubkey =>\n        staged = builtin schnorr_get_address staged_pubkey;\n        staged_is_sender = builtin eq _sender staged;\n        match staged_is_sender with\n        | False => e = NotAuthorized; ThrowError e\n        | True => \n            admin := staged;\n            admin_pubkey := staged_pubkey\n        end\n    end\nend\n\nprocedure SetWinTier(tier: Uint128Pair)\n    match tier with\n    | Uint128Pair cur_tier amt =>\n        win_tiers[cur_tier] := amt\n    end\nend\n\n\ntransition SetWinTiers(tiers: List Uint128Pair)\n    IsAdmin;\n    win_tiers := empty_uint128_uint128_map;\n    forall tiers SetWinTier\nend\n\n(* to withdraw funds once the contract has too much state *)\ntransition Send(to: ByStr20, amt: Uint128)\n    IsAdmin;\n    SendZil to amt\nend\n\ntransition UpdateFeeAddr(new: ByStr20)\n    IsAdmin;\n    fee_addr := new\nend\n\ntransition UpdateTicketPrice(new: Uint128)\n    IsAdmin;\n    ticket_price := new\nend\n\ntransition UpdateFeeCut(new: Uint128)\n    IsAdmin;\n    fee_cut := new\nend\n\n(* user transitions *)\n\nprocedure AcceptAsTickets()\n    cur_balance <- _balance;\n    accept;\n    accepted_balance <- _balance;\n    gain = builtin sub accepted_balance cur_balance;\n    price <- ticket_price;\n    to_send_back = builtin rem gain price;\n    ticket_num = builtin div gain price;\n    is_zero_tickets = builtin eq ticket_num zero;\n    match is_zero_tickets with\n    | True => e = NotEnoughZILSent; ThrowError e\n    | False =>\n        (* save tmp ticket number for _sender *)\n        tmp_ticket_amt := ticket_num;\n        is_zero_to_send_back = builtin eq to_send_back zero;\n        match is_zero_to_send_back with\n        | True => (* do nothing *)\n        | False =>\n            (* refund overpay *)\n            SendZil _sender to_send_back\n        end\n    end\nend\n\nprocedure GiveTickets(to: ByStr20, amt: Uint128)\n    maybe_spins <- players_spins[to];\n    spins = optional_uint128 maybe_spins;\n    spins_added = builtin add spins amt;\n    players_spins[to] := spins_added\nend\n\nprocedure AssertChequeValidityAndVoidIt(cheque_hash: ByStr, pubkey: ByStr33, signature: ByStr64)\n    AssertChequeHashValid cheque_hash;\n    valid_sig = builtin schnorr_verify pubkey cheque_hash signature;\n    match valid_sig with\n        | True =>\n            (* only here the message is valid! *)\n            DoVoidCheque cheque_hash\n        | False => e = InvalidSignature; ThrowError e\n    end\nend\nprocedure AssertIsAdminSignatureAndVoidIt(signable: Signable, signature: ByStr64)\n    cheque_hash = signable_to_cheque_hash signable;\n    admin_pub_temp <- admin_pubkey;\n    AssertChequeValidityAndVoidIt cheque_hash admin_pub_temp signature\nend\nprocedure IncrementClaim(target: ByStr20)\n    maybe_ticket_max <- players_spins[target];\n    ticket_max = optional_uint128 maybe_ticket_max;\n    (* increment claim make sure that claim >= ticket *)\n    maybe_claimed_index <- players_claimed[target];\n    claimed_index = optional_uint128 maybe_claimed_index;\n    claimed_incremented = builtin add claimed_index one;\n    (* make sure claimed incremented is not bigger than ticket_max *)\n    claim_is_bigger = uint128_gt claimed_incremented ticket_max;\n    match claim_is_bigger with\n    | True => e = CannotClaimMoreThanMax; ThrowError e\n    | False =>\n        players_claimed[target] := claimed_incremented\n    end\nend\n\nprocedure AssertDiscountNotBiggerThanMax(d: Uint128)\n    is_bigger = uint128_gt d discount_denominator;\n    match is_bigger with\n    | True => e = DiscountTooBig; ThrowError e\n    | False =>\n    end\nend\n\n\nprocedure SendWinnings(target: ByStr20, win_tier: Uint128, amt: Uint128, fee: Uint128)\n    e = {_eventname : \"Won\"; target: target; amt: amt; win_tier: win_tier; fee_nominator: fee};\n    event e;\n    SendZil target amt\nend\n\n\nprocedure PayFee(amt: Uint128)\n    fund <- fee_addr;\n    SendZil fund amt\nend\n\n\nprocedure PayOutReward(target: ByStr20, win_tier: Uint128, discount: Uint128)\n    maybe_win_amt <- win_tiers[win_tier];\n    match maybe_win_amt with\n    | None => e = UnknownRewardTier; ThrowError e\n    | Some win_amt =>\n        is_zero = builtin eq win_amt zero;\n        match is_zero with\n        | True => (* do nothing is fine *)\n        | False =>\n            (* do reward payout and reward fee *)\n            AssertDiscountNotBiggerThanMax discount;\n            discount_is_max = builtin eq discount discount_denominator;\n            match discount_is_max with\n            | True => SendWinnings target win_tier win_amt zero\n            | False =>\n                fee_cut_local <- fee_cut;\n                res1 = sub_percentage_of_amt fee_cut_local discount;\n                match res1 with\n                | Uint128Pair fee_discounted asdf =>\n                    result = sub_percentage_of_amt win_amt fee_discounted;\n                    match result with\n                    | Uint128Pair reward fee =>\n                        SendWinnings target win_tier reward fee_discounted;\n                        PayFee fee\n                    end\n                end\n            end\n        end\n    end\nend\n\ntransition AddFunds()\n    AcceptAsTickets;\n    (* get tickets to give and reasign *)\n    ticket_amt <- tmp_ticket_amt;\n    tmp_ticket_amt := zero;\n    GiveTickets _sender ticket_amt;\n    (* emit bought tickets *)\n    e = {_eventname : \"BoughtTickets\"; sender: _sender; ticket_amt: ticket_amt};\n    event e\nend\n\nprocedure ClaimSpin(spin: SpinClaim)\n    target <- tmp_target;\n    match spin with\n    | SpinClaim discount spin_num win_tier signature =>\n        signable = Signable target discount spin_num win_tier _this_address;\n        AssertIsAdminSignatureAndVoidIt signable signature;\n        (* since we got here that means that the spin meta tx is valid *)\n        (* now we need to incremet claim and pay out rewards *)\n        IncrementClaim target;\n        PayOutReward target win_tier discount\n    end\nend\n\n(* \n    @dev: have to sign target discount spinNumber winTier _this_addr\n    _sender can be anybody these are meta transactions\n*)\ntransition ClaimSpins(target: ByStr20, spins: List SpinClaim)\n    tmp_target := target;\n    forall spins ClaimSpin\nend\n";
export default _default;
